######################################################################
###### Jekel, M., Fiedler, S., & Glckner, A. Diagnostic Task ########
###### Selection for Strategy Classification in Judgment and  ########
###### Decision Making ###############################################
######################################################################

EDTS = function(

#### working directory

setWorkingDirectory = "c:/EDTS_v2.0/",

#### EDTS parameters

validities = c(.80,.70,.60,.55),
measures = c("choice", "time", "confidence"),
rescaleMeasures = c(0,1,1),
weightingMeasures = c(1, 1, 1),
strategies = c("PCS", "TTB", "EQW", "WADDcorr", 
"RAND", "RAT", "WADDuncorr"),
generateTasks = 1,
reduceSetOfTasks = 1,
derivePredictions = 1,
printStatus = 1,
saveFiles = 1,
setOfTasks = "none",
distanceMetric = "Euclidian",
loadFunctions = 1,

#### PCS parameters

PCSdecay = .05,
PCSfloor = -1,
PCSceiling = 1,
PCSstability = 10^6,
PCSsubtrahendResc = .5,
PCSfactorResc = 1,
PCSexponentResc = 2
)
{

#### set R working directory

setwd(setWorkingDirectory)

#### define variables

numbOfCues = length(validities)

######################################################################
#################### generate/load cue environment  ##################
######################################################################

#### generate cue environment 

if(generateTasks == 1 & length(setOfTasks) == 1){

if(printStatus == 1){

print("####################################################")
print("############## generating/loading environment ######")
print("####################################################")
}

source("taskGenerator.r")

patternQualified = taskGenerator(reduceSetOfTasks, numbOfCues,
saveFiles= saveFiles)

} else { 

#### load cue environment

if (length(setOfTasks) == 1){

patternQualified = as.matrix(read.csv("tasks.csv"))

} else {

patternQualified = setOfTasks
}
}

if(printStatus == 1){

print("done")
}

######################################################################
#################### derive predictions ##############################
######################################################################

if(printStatus == 1){

print("####################################################")
print("############## deriving/loading predictions ########")
print("####################################################")
}

numbOfCuePatterns = max(patternQualified[,1])

#### load function for strategies

if (loadFunctions == 1){

source("strategies/PCSv2.r")
source("strategies/transformStandardPCS.r")
source("strategies/TTB.r")
source("strategies/EQW.r")
source("strategies/WADDcorr.r")
source("strategies/RAT.r")
source("strategies/WADDuncorr.r")
}

#### Predictions

## define prediction matrices

PCSPred = matrix(NA,numbOfCuePatterns,3)
TTBPred = matrix(NA,numbOfCuePatterns,3)
EQWPred = matrix(NA,numbOfCuePatterns,3)
WADDcorrPred = matrix(NA,numbOfCuePatterns,3)
RandomPred = matrix(NA,numbOfCuePatterns,3)
RATPred = matrix(NA,numbOfCuePatterns,3)
WADDuncorrPred = matrix(NA,numbOfCuePatterns,3)

## derive predictions

if(derivePredictions == 1){

for (itemLoop in 1 : numbOfCuePatterns){

#### PCS

if ("PCS" %in% strategies == TRUE){

matrixPCS = transPCSpairwiseComp(
(PCSfactorResc *(validities-PCSsubtrahendResc))^PCSexponentResc , 
.01 * 
patternQualified[patternQualified[,1] == itemLoop,2:3],
numbCues = numbOfCues)

PCSout = PCSv2(
activ = c(1,rep(0,numbOfCues),0,0),
weightsNet = matrixPCS,
decay = PCSdecay,
flo = PCSfloor,
ceil = PCSceiling,
stability = PCSstability)

PCSresul = ifelse(round(PCSout[length(PCSout)],6) > 
round(PCSout[length(PCSout)-1],6), 0,1)

PCSresul = ifelse(round(PCSout[length(PCSout)],6) == 
round(PCSout[length(PCSout)-1],6), 0.5,PCSresul)

PCSresul = c(PCSresul, PCSout[1], abs(PCSout[length(PCSout)-1] -
PCSout[length(PCSout)]))

PCSPred[itemLoop, ] = PCSresul
}

#### Predictions TTB

if ("TTB" %in% strategies == TRUE){

TTBPred[itemLoop, ] = TTB(
validities = validities,
cuePattern = 
patternQualified[patternQualified[,1] == itemLoop, 2:3]
)
}

#### Predictions EQW

if ("EQW" %in% strategies == TRUE){

EQWPred[itemLoop, ] = EQW(
validities = validities,
cuePattern = 
patternQualified[patternQualified[,1] == itemLoop, 2:3]
)
}

#### Predictions WADDcorr

if ("WADDcorr" %in% strategies == TRUE){

WADDcorrPred[itemLoop, ] = WADDcorr(
validities = validities,
cuePattern = 
patternQualified[patternQualified[,1] == itemLoop, 2:3]
)
}

#### Predictions RAND

if ("RAND" %in% strategies == TRUE){

RandomPred[itemLoop, ] = c(1/2,0,0)
}

#### Predictions RAT

if ("RAT" %in% strategies == TRUE){

RATPred[itemLoop, ] = RAT(
validities = validities,
cuePattern = 
patternQualified[patternQualified[,1] == itemLoop, 2:3]
)
}

#### Predictions WADDuncorr

if ("WADDuncorr" %in% strategies == TRUE){

WADDuncorrPred[itemLoop, ] = WADDuncorr(
validities = validities,
cuePattern = 
patternQualified[patternQualified[,1] == itemLoop, 2:3]
)
}

#

if(printStatus == 1){

print(paste("predictions for task #",itemLoop,"of",
numbOfCuePatterns,"derived"))
}
}

totalPredictions = 
cbind(
1 : numbOfCuePatterns,
PCSPred,
TTBPred,
EQWPred,
WADDcorrPred,
RandomPred,
RATPred,
WADDuncorrPred)

colnames(totalPredictions) = 
c("Task.Number",
paste(rep(c("PCS.", "TTB.", "EQW.", "WADDcorr.", 
"RAND.", "RAT.", "WADDuncorr."),
each = 3), c("choice","time","confidence"), sep=""))

}else{

totalPredictions = as.matrix(read.csv("predictions.csv"))
}

totalPredictions = 
totalPredictions[,is.na(colSums(totalPredictions)) == FALSE]

###################################################################### 
################ identify the number of measures, strategies etc. ####
######################################################################

namesPred = colnames(totalPredictions)

indexStratCheck = c(2, 2, rep(c(1,0), (length(namesPred)-1)))

## determine number of strategies and number of measures

namesStrategies = (unlist(strsplit(namesPred, ".", fixed = T))
[indexStratCheck == 1])

namesMeasures = (unlist(strsplit(namesPred, ".", fixed = T))
[indexStratCheck == 0])

numbOfCuePatterns = NROW(totalPredictions)

## select strategies and measures for EDTS

strategiesIncludedForEDTS = namesStrategies %in% strategies

measuresIncludedForEDTS = namesMeasures %in% measures

namesStrategiesIncludedForEDTS = 
unique(namesStrategies[strategiesIncludedForEDTS])

namesMeasuresIncludedForEDTS = 
unique(namesMeasures[measuresIncludedForEDTS])

numberOfMeasures = length(namesMeasuresIncludedForEDTS)

numberOfStrategies = length(namesStrategiesIncludedForEDTS)

include = colSums(rbind(strategiesIncludedForEDTS,
measuresIncludedForEDTS)) == 2

## default weighting

if(length(weightingMeasures) != numberOfMeasures){

weightingMeasures = rep(1, numberOfMeasures)
}

totalPredictions = 
totalPredictions[ , 
c(TRUE, include)]

## save predictions

if(saveFiles == 1 & derivePredictions == 1){

write.csv(totalPredictions,"predictions.csv", row.names = F)
}

if(printStatus == 1){

print("done")

print("####################################################")
print("############## applying EDTS #######################")
print("####################################################")
print("#")

}

######################################################################
#################### EDTS ############################################
######################################################################

#### Step 1: Generate standardized prediction vectors

## formula 5: Rescaling of prediction vectors per strategy

predictionsRescaled = 
t((t(totalPredictions) - apply(totalPredictions,2,min)) /
(apply(totalPredictions,2,max) - apply(totalPredictions,2,min)))

# do not rescale measures as indicated by rescaleMeasures

if(numberOfMeasures != length(rescaleMeasures)){

rescaleMeasures = rep(1, numberOfMeasures)
}

rescaleMeasuresIndex = c(999, rep(rescaleMeasures,
numberOfStrategies))

predictionsRescaled[, rescaleMeasuresIndex == 0] = 
totalPredictions[, rescaleMeasuresIndex == 0]

predictionsRescaled = ifelse(is.na(predictionsRescaled), 0, 
predictionsRescaled)

predictionsRescaled[, 1] = 1 : numbOfCuePatterns

if(printStatus == 1){

print("############################################")
print("############### set of strategies ##########")
print("############################################")
print(namesStrategiesIncludedForEDTS)
print("############################################")
print("############### number of tasks ############")
print("############################################")
print(NROW(totalPredictions))
print("############################################")
print("############### set of measures ############")
print("############################################")
print(namesMeasuresIncludedForEDTS)
print("############################################")
print("############### rescaled measures ##########")
print("############################################")

if (length(namesMeasuresIncludedForEDTS[rescaleMeasures == 1]) > 0){

print(namesMeasuresIncludedForEDTS[rescaleMeasures == 1])
} else {

print("none")
}

print("############################################")
print("############### weighting measures #########")
print("############################################")

print(paste(namesMeasuresIncludedForEDTS,": ",weightingMeasures,
sep = ""))

}

#### Step 2: Calculate diagnosticity scores for strategy comparisons

## formula 6: Diagnosticity Scores

# all possible unique strategy comparisons

stratComparisons = combn(1: numberOfStrategies, 2)

ED = matrix(NA, numbOfCuePatterns, NCOL(stratComparisons))

stratCoding = 
rep(1: numberOfStrategies, each = numberOfMeasures)

stratCoding = c(max(stratCoding) + 1, stratCoding)

#

for (loopEDTS in 1 : NCOL(stratComparisons)){

if (distanceMetric == "Euclidian"){

ED[,loopEDTS] = 

sqrt(rowSums(cbind((t(weightingMeasures * 
t(predictionsRescaled[ , stratCoding == 
stratComparisons[1,loopEDTS]] -
predictionsRescaled[ , stratCoding == 
stratComparisons[2,loopEDTS]])))^2,0)))
}

if (distanceMetric == "Taxicab"){

ED[,loopEDTS] = 

rowSums(cbind(abs(t(weightingMeasures * 
t(predictionsRescaled[ , stratCoding == 
stratComparisons[1,loopEDTS]] -
predictionsRescaled[ , stratCoding == 
stratComparisons[2,loopEDTS]]))),0))
}

}

# formula 7

EDRescaled = t((t(ED) - apply(ED, 2, min))/
(apply(ED, 2, max) - apply(ED, 2, min)))

EDRescaled = ifelse(is.na(EDRescaled) == TRUE, ED, EDRescaled)

#### Step 3: Calculate the average, 
#### minimum, and maximum diagnosticity scores

# formula 8

ADMean = apply(EDRescaled, 1, mean)
ADMin = apply(EDRescaled, 1, min)
ADMax = apply(EDRescaled, 1, max)
ADMedian = apply(EDRescaled, 1, median)

##### save diagnosticity scores

ADcomplete = 
cbind(1 : numbOfCuePatterns,
ADMean, ADMin, ADMax, ADMedian,
EDRescaled
)

namesComparisons = paste("Diag",
namesStrategiesIncludedForEDTS[stratComparisons[1,]],
namesStrategiesIncludedForEDTS[stratComparisons[2,]], sep = ".")

colnames(ADcomplete) = c(
"Task.Number",
"AD", "Diag.Min", "Diag.Max", "Diag.Median",
namesComparisons)

if (saveFiles == 1){

write.csv(ADcomplete,"outputEDTS.csv", row.names = F)
}

if(printStatus == 1){

print("####################################################")
print("##################### done #########################")
print("####################################################")
}

return(list("tasks" = patternQualified, "validities" = validities, 
"predictions" = totalPredictions, "EDTS" = ADcomplete))
}

###

